/*
**  Enduro Result Layout (in seconds)
**  JavaScript engine
*/


/* Import needed packages */
importPackage( java.util );

/* Static variable defines */
var PluginType_Track = 1;
var PluginType_Route = 2;
var PluginType_Both = 3;
var PluginType_Http = 4;

var PluginOutput_Html = 1;
var PluginOutput_List = 2;


/* Get short presentation of plugin's name */
function getName()
{
  return "Enduro Result Layout (in seconds)";
}

/* Get plugin's version number */
function getVersion()
{
  return 1.0;
}

/* Get long presentation of plugin's name */
function getFullName()
{
  return getName() + " v" + getVersion();
}

/* Default implementation of method toString */
function toString()
{
  return getFullName();
}

/* Get plugin type */
function getType()
{
  //return PluginType_Track;
  return PluginType_Route;
  //return PluginType_Both;
  //return PluginType_Http;
}

/* Get plugin output format */
function getOutputFormat()
{
  //return PluginOutput_List;
  return PluginOutput_Html;
}

/* Get possible options displayed at plugin window combobox */
function getOptionGroup()
{
  return Arrays.asList( [ ] );
}

/* Get list column names when using PluginOutput_List */
function getColumns()
{
  return Arrays.asList( [ ] );
}

/* Get information about data what we need with next invoke (fireDataUpdate) */
/* See Plugin API for available methods */
function getInvokeWith()
{
  return Arrays.asList( [ 'getEnduroGapTimes', 'getRounds', 'getCompetitionResults' ] );
}

/* Invoke event when competitor is selected in list when using PluginOutput_List */
function fireCompetitorSelection( competitors )
{
  //
}

function StringToTime( string )
{
  var negative = false;
  if( string.charAt( 0 ) == '-' )
    negative = true;

  if( string == null || string == "00:00:00.000" )
    return 0;
  else if( string.match("^([0-2]{1})([0-9]{1})(:)([0-5]{1})([0-9]{1})(:)([0-5]{1})([0-9]{1})(\\.)([0-9]{3})$") ||
           string.match("^\\-([0-2]{1})([0-9]{1})(:)([0-5]{1})([0-9]{1})(:)([0-5]{1})([0-9]{1})(\\.)([0-9]{3})$")
         )
  {
    var pString = string.split(":");
    var sString = pString[2].split("\\.");

    var sum = ( pString[0] * 3600 * 1000 );
    sum += ( pString[1] * 60 * 1000 );
    sum += ( sString[0] * 1000 );
    sum += ( sString[1] * 1 );

    if( negative )
      sum = -sum;

    return sum;
  }
  else
    return 0;
}

function TimeToString( time )
{
  var negative = false;
  if( time < 0 )
  {
    negative = true;
    time = -time;
  }

  var hours = ( time / ( 3600 * 1000 ) );
  var minutes = ( time % ( 3600 * 1000 ) ) / ( 60 * 1000 );
  var seconds = ( ( time % ( 3600 * 1000 ) ) % ( 60 * 1000 ) ) / 1000;
  var sSeconds = ( ( time % ( 3600 * 1000 ) ) % ( 60 * 1000 ) ) % 1000;
  
  var tHours = ( Math.round( Math.floor( hours ) ) ) + "";
  var tMinutes = ( Math.round( Math.floor( minutes ) ) ) + "";
  var tSeconds = ( Math.round( Math.floor( seconds ) ) ) + "";
  var tSSeconds = ( Math.round( Math.floor( sSeconds ) ) ) + "";
  
  if( hours < 10 )
    tHours = "0" + tHours;
  if( minutes < 10 )
    tMinutes = "0" + tMinutes;
  if( seconds < 10 )
    tSeconds = "0" + tSeconds;
  
  if( sSeconds < 10 )
    tSSeconds = "00" + tSSeconds;
  else if( sSeconds < 100 )
    tSSeconds = "0" + tSSeconds;

  return ( ( negative ) ? "-" : "" ) + tHours + ":" + tMinutes + ":" + tSeconds + "." + tSSeconds;
}

function countOccurences( base, searchFor )
{
  var len = searchFor.length;
  var result = 0;

  if( len > 0 )
  {  // search only if there is something
    var start = base.indexOf( searchFor );
    while( start != -1 )
    {
      result++;
      start = base.indexOf( searchFor, start + len );
    }
  }
  return result;
}

function TimeToSeconds( time )
{
  var secs = StringToTime( time ) / 1000.0;
  return secs.toFixed(2);
}

function by( i, dir )
{
  return function( a, b ){ a = a[i]; b = b[i]; return a.toLowerCase() == b.toLowerCase() ? 0 : ( a.toLowerCase() < b.toLowerCase() ? -1 * dir : dir ) }
}

function oc( a )
{
  var o = {};

  for( var i=0; i < a.length; i++ )
  {
    o[ a[i] ] = '' ;
  }

  return o;
}

/* Invoke function (data processing itself) */
function fireDataUpdate( invokeData, selectedOptionIndex )
{
  var invokes = invokeData.toArray();
  
  var enduroGapTimes = invokes[ getInvokeWith().indexOf("getEnduroGapTimes") ].toArray();
  var rounds = invokes[ getInvokeWith().indexOf("getRounds") ].toArray();
  var competitionResults = invokes[ getInvokeWith().indexOf("getCompetitionResults") ].toArray();
  
  var html = "";
  html += "<table style=\"width: 100%; font-family: sans-serif;\">\n";

  var roundNameBySequence = [];

  for( var i=0; i < rounds.length; i++ )
  {
    round = rounds[i].toArray();
    roundNameBySequence[ round[3] ] = round[1]; // round [3]: sequence  // round [1]: name
  }

  var gapsHeader = [];
  var gapsHeaderCount = 0;

  var gaps = [];
  var counts = [];

  var old = "";
  var put = "";

  for( var i=0; i < enduroGapTimes.length; i++ )
  {
    enduroGapTime = enduroGapTimes[i].toArray();

    old = "";
    if( gaps[ enduroGapTime[2] ] ) // enduroGapTime [2]: number
      old = gaps[ enduroGapTime[2] ];

    if( counts[ enduroGapTime[2] ] )
      counts[ enduroGapTime[2] ] = counts[ enduroGapTime[2] ] + 1;
    else
      counts[ enduroGapTime[2] ] = 0;

    put = "";
    if( counts[ enduroGapTime[2] ] > 5 )
    {
      counts[ enduroGapTime[2] ] = 0;
      put += "<tr style='color: gray; text-decoration: italic;'>";
    }

    put += old + "<td style=\"border-bottom: 1px solid silver;\">" + TimeToSeconds( enduroGapTime[7] ) + "</td>"; // enduroGapTime [7]: gap

    if( !( roundNameBySequence[ enduroGapTime[6] ] in oc( gapsHeader ) ) ) // enduroGapTime [6]: lap
    {
      gapsHeader[ gapsHeaderCount ] = roundNameBySequence[ enduroGapTime[6] ];
      gapsHeaderCount ++;
    }

    gaps[ enduroGapTime[2] ] = put;
  }

  var rowCounter = 0;
  var previousTime = "00:00:00.000";
  var previousClass = "";

  var competitionResultsNew = [];
  var competitionResultsNewCounter = 0;

  for( var i=0; i < competitionResults.length; i++ )
  {
    competitionResultOld = competitionResults[i].toArray();

    competitionResultsNew[ competitionResultsNewCounter ] = competitionResultOld;
    competitionResultsNewCounter ++;
  }

  competitionResultsNew.sort( by( 6, 1 ) ); // [6]: total (asc)
  competitionResultsNew.sort( by( 7, -1 ) ); // [7]: laps (desc)
  /*competitionResultsNew.sort( by( 8, 1 ) ); // [8]: penalties (asc)*/
  competitionResultsNew.sort( by( 4, 1 ) ); // [4]: class (asc)

  for( var i=0; i < competitionResultsNew.length; i++ )
  {
    competitionResult = competitionResultsNew[i];

    if( "" + previousClass != "" + competitionResult[4] ) // [4]: class
    {
      html += "<tr><td colspan='7'><h3>" + competitionResult[4] + "</h3></td></tr>";
      html += "<tr>";
      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Position</td>";
      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Number</td>";
      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Name</td>";
      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Additional</td>";

      for( var h=0; h < gapsHeader.length; h++ )
      {
        html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">" + gapsHeader[ h ] + "</td>";
      }

      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Start</td>";
      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Penalties</td>";
      html += "<td style=\"border-bottom: 1px solid gray; padding-right: 15px; background-color: #F5F5F5; font-weight: bold;\">Total</td>";
      html += "</tr>\n";

      rowCounter = 0;
    }

    rowCounter ++;
    var diff = TimeToString( StringToTime( competitionResult[6] ) - StringToTime( previousTime ) ); // [6]: total


    var penalty = "0";
    var startPenalty = "0";
    var printPos = true;

    var penaltySecs = StringToTime( competitionResult[9] ); // [9]: penalties_time
    var dnsSecs = StringToTime( competitionResult[10] ); // [10]: dns_time
    penalty = penaltySecs / 1000.0;
    penalty = penalty.toFixed(0);

    startPenalty = dnsSecs / 1000.0;
    startPenalty = startPenalty.toFixed(0);

    if( penaltySecs == 1000*60*120 )
    {
      penalty = "OTA";
      printPos = false;
    }
    else if( penaltySecs == 1000*60*180 )
    {
      penalty = "RET.D";
      printPos = false;
    }
    else if( penaltySecs == 1000*60*240 )
    {
      penalty = "EXC";
      printPos = false;
    }

    if( diff == competitionResult[6] )
      diff = "-";
    else if( diff.indexOf( "-" ) > -1 )
      diff = "-";

    html += "<tr>";

    if( competitionResult[8] == "" && printPos ) // [8]: penalties
      html += "<td style=\"border-bottom: 1px solid silver;\">" + rowCounter + ".</td>";
    else
      html += "<td style=\"border-bottom: 1px solid silver;\"></td>";

    html += "<td style=\"border-bottom: 1px solid silver;\">" + competitionResult[2] + "</td>"; // [2]: number
    html += "<td style=\"border-bottom: 1px solid silver;\">" + competitionResult[0] + "</td>"; // [0]: name
    html += "<td style=\"border-bottom: 1px solid silver;\">" + competitionResult[1] + "</td>"; // [1]: additional

    var gapsString = "";

    if( gaps[ competitionResult[2] ] )
    {
      gapsString = gaps[ competitionResult[2] ];
      html += gapsString;
    }

    for( var l=0; l < ( gapsHeader.length - countOccurences( gapsString, "<td" ) ); l++ )
      html += "<td style=\"border-bottom: 1px solid silver;\"></td>";

    html += "<td style=\"border-bottom: 1px solid silver;\">" + startPenalty + "</td>";
    html += "<td style=\"border-bottom: 1px solid silver;\">" + penalty + "</td>";

    if( printPos )
      html += "<td style=\"border-bottom: 1px solid silver;\">" + TimeToSeconds( competitionResult[6] ) + "</td>";
    else
      html += "<td style=\"border-bottom: 1px solid silver;\"></td>";

    html += "</tr>\n";

    previousTime = competitionResult[6];
    previousClass = competitionResult[4]; // [4]: class
  }

  html += "</table>\n";

  return html;
}

